﻿using Win8Contracts.Data;

using System;
using System.IO;
using System.Collections.Generic;
using Windows.ApplicationModel.DataTransfer;
using Windows.Graphics.Printing;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Printing;

// The Item Detail Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234232

namespace Win8Contracts
{
	/// <summary>
	/// A page that displays details for a single item within a group while allowing gestures to
	/// flip through other items belonging to the same group.
	/// </summary>
	public sealed partial class ItemDetailPage : Win8Contracts.Common.LayoutAwarePage
	{
		private PrintDocument _printDocument;
		private IPrintDocumentSource _printDocumentSource;

		public ItemDetailPage()
		{
			this.InitializeComponent();
		}

		protected override void OnNavigatedTo(Windows.UI.Xaml.Navigation.NavigationEventArgs e)
		{
			// setup for sharing
			DataTransferManager.GetForCurrentView().DataRequested += DataRequested;

			// setup for printing
			_printDocument = new PrintDocument();
			_printDocumentSource = _printDocument.DocumentSource;
			_printDocument.GetPreviewPage += GetPrintPreviewPage;
			_printDocument.AddPages += AddPrintPages;

			PrintManager printMan = PrintManager.GetForCurrentView();
			printMan.PrintTaskRequested += PrintTaskRequested;

			base.OnNavigatedTo(e);
		}

		protected override void OnNavigatingFrom(Windows.UI.Xaml.Navigation.NavigatingCancelEventArgs e)
		{
			DataTransferManager.GetForCurrentView().DataRequested -= DataRequested;

			PrintManager.GetForCurrentView().PrintTaskRequested -= PrintTaskRequested;

			base.OnNavigatingFrom(e);
		}

		private async void DataRequested(DataTransferManager sender, DataRequestedEventArgs e)
		{
			// when doing anything async in a DataRequested event, must get and complete a deferral!
			DataRequestDeferral deferral = e.Request.GetDeferral();

			try
			{
				SampleDataItem item = (this.flipView.SelectedItem as SampleDataItem);
				if(item != null)
				{
					Uri uri = new Uri(item.ImagePath);

					// stream used for thumbnails and bitmap share
					RandomAccessStreamReference stream = RandomAccessStreamReference.CreateFromUri(uri);

					// StorageFile used for storage items share
					StorageFile file = await StorageFile.CreateStreamedFileFromUriAsync(Path.GetFileName(item.ImagePath), uri, stream);
					List<IStorageItem> items = new List<IStorageItem> { file };

					// setup appropriate sharing properties
					e.Request.Data.Properties.Title = "Sharing " + item.Title;
					e.Request.Data.Properties.Description = "by " + item.Subtitle;
					e.Request.Data.Properties.Thumbnail = stream;

					// use both StorageItem and Bitmap so we can share to more apps
					e.Request.Data.SetStorageItems(items);
					e.Request.Data.SetBitmap(stream);
				}
			}
			finally
			{
				// must always call complete!
				deferral.Complete();
			}
		}

		private void PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs args)
		{
			var task = args.Request.CreatePrintTask("Print Flickr Photo", sourceRequested => sourceRequested.SetSource(_printDocumentSource));
			task.Options.Orientation = PrintOrientation.Landscape;
		}

		private void AddPrintPages(object sender, AddPagesEventArgs e)
		{
			_printDocument.AddPage(this.Content);
			_printDocument.AddPagesComplete();
		}

		private void GetPrintPreviewPage(object sender, GetPreviewPageEventArgs e)
		{
			_printDocument.SetPreviewPage(e.PageNumber, this.Content);
		}

		/// <summary>
		/// Populates the page with content passed during navigation.  Any saved state is also
		/// provided when recreating a page from a prior session.
		/// </summary>
		/// <param name="navigationParameter">The parameter value passed to
		/// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
		/// </param>
		/// <param name="pageState">A dictionary of state preserved by this page during an earlier
		/// session.  This will be null the first time a page is visited.</param>
		protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
		{
			// Allow saved page state to override the initial item to display
			if (pageState != null && pageState.ContainsKey("SelectedItem"))
			{
				navigationParameter = pageState["SelectedItem"];
			}

			// TODO: Create an appropriate data model for your problem domain to replace the sample data
            var item = SampleDataSource.GetItem((String)navigationParameter);
            this.DefaultViewModel["Group"] = item.Group;
            this.DefaultViewModel["Items"] = item.Group.Items;
            this.flipView.SelectedItem = item;
		}

		/// <summary>
		/// Preserves state associated with this page in case the application is suspended or the
		/// page is discarded from the navigation cache.  Values must conform to the serialization
		/// requirements of <see cref="SuspensionManager.SessionState"/>.
		/// </summary>
		/// <param name="pageState">An empty dictionary to be populated with serializable state.</param>
		protected override void SaveState(Dictionary<String, Object> pageState)
		{
			var selectedItem = (SampleDataItem)this.flipView.SelectedItem;
			pageState["SelectedItem"] = selectedItem.UniqueId;
		}
	}
}
